//+------------------------------------------------------------------+
//|                                                DI_Cross_Rev3.mq4 |
//|                                                  Frans Hammersma |
//|                                                            http: |
//+------------------------------------------------------------------+
#property copyright "Frans Hammersma"
#property link      "http:"
//---- input parameters

extern int        ADXperiod=14;//3;
extern int        HighPlus = 2;
extern int        LowPlus = 2; 
extern int        HorizontaleLijn=90;
extern double        ADXverschil = 3.0;
extern int        StopLoss= 50;
extern int        TakeProfit=100;
extern double     Lots=0.1;
extern int        StartHour =4;
extern int        EntryAddPips =10;
// constant "off market"
#define OP_BALANCE 6


// settings for trading
extern int  TradeDay                   = 0; 
extern bool ReversDay                  = false;

// trading system reverse
extern bool ReversTradeSignal          = false;

//  trading frequency settings
extern bool TradeSignalEveryTick       = false;
extern int  TradeSignalBarPeriod       = 0;

//  order modification settings
extern bool ModifyMarketOrderEveryTick = false;
extern bool ModifyPendingEveryTick     = false;
extern int  ModifyMarketBarPeriod      = 0;
extern int  ModifyPendingBarPeriod     = 0;

// 
extern int  Slippage                   = 3; // slippage
extern int  ExpertMagicNumber          = 0; // specify the necessary number  


double Tickets[][9];// array to store information about "friendly" orders:
// Tickets[][0] - ticket number
// Tickets[][1] - order  type
// Tickets[][2] - lots
// Tickets[][3] - open price
// Tickets[][4] - Stop Loss
// Tickets[][5] - TakeProfit
// Tickets[][6] - MagicNumber
// Tickets[][7] - expiration time
// Tickets[][8] - open time
// 

double newSL_and_TP[][5];// array to store the new values of SL and TP
// newSL_and_TP[][0] - ticket number to control
// newSL_and_TP[][1] - new value of SL
// newSL_and_TP[][2] - new value of TP
// newSL_and_TP[][3] - new open price (for pending orders)
// newSL_and_TP[][4] - 0 for open orders and 1 for pending orders)

string CommentsTicket[100][2]; //array of symbols and comments on orders, 100 lines would be enough
// CommentsTicket[][0] - symbol name
// CommentsTicket[][1] - comment on the order

//+-----------------------------------------------------------------------------+
//|  if the period is not specified correctly, it returns the current period    |
//+-----------------------------------------------------------------------------+
int getCorrectTimeFrame(int period)
   {
   int res=period;
//----
   switch (period)
      {
      case PERIOD_D1:  break;  // permitted timeframe, do nothing
      case PERIOD_H4:  break;  // permitted timeframe, do nothing
      case PERIOD_H1:  break;  // permitted timeframe, do nothing
      case PERIOD_M30: break;  // permitted timeframe, do nothing
      case PERIOD_M15: break;  // permitted timeframe, do nothing
      case PERIOD_M5:  break;  // permitted timeframe, do nothing
      case PERIOD_M1:  break;  // permitted timeframe, do nothing
      default: res=Period();   // wrong timeframe, set by default
      }
//----
   return(res);      
   }
//+------------------------------------------------------------------+
//| it produces a message about EA settings                          |
//+------------------------------------------------------------------+
void StartMessage()
   {
   int i=0;
   string currString="";
   string array[3];
//----
   array[0]=StringConcatenate("Expert Advisor ",WindowExpertName()," Heeft de volgende instellingen:");

   if (TradeSignalEveryTick)
      array[1]="1)trade signals are taken at every tick; ";
   else       
      array[1]=StringConcatenate("1)trade signals are taken at every bar with the period of ",TradeSignalBarPeriod," minutes;");

   if (TradeDay==0)   // trade day is not specified
      array[2]="2)everyday's trading is allowed; ";
   else 
      {
      if (ReversDay) //  trading isn't allowed on the given day
         array[2]=StringConcatenate("2)trading is allowed on all days except day number ",TradeDay);
      else           //  trading is allowed only on the given day
         array[2]=StringConcatenate("2)trading is allowed only on day number ",TradeDay);
      }
   
   
   
   for ( i=0;i<3;i++) currString=StringConcatenate(currString,"\n",array[i]);
   Comment(currString);

   for (i=2;i>=0;i--) Print(array[i]);
   
   
//----
   }
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   TradeSignalBarPeriod=getCorrectTimeFrame(TradeSignalBarPeriod);   
   StartMessage(); // remind the EA settings
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+

   
  
//+------------------------------------------------------------------+
//|  trade signals producing function                                |
//+------------------------------------------------------------------+






int yourFunction(int workPeriod)

   {
   int res=OP_BALANCE;
   
//----
//  here there must be your code for producing trade signals considering timeframe workPeriod)
   
   
   
      int total = OrdersTotal();
      double high_val, Low_val, order_priceHigh, order_priceLow;

   if ((Hour()==StartHour) && (Volume[0]==1)) // go for it on first tick of the new bar
    
      
      
      high_val=High[iHighest(NULL,PERIOD_H1,MODE_HIGH,4,1)]; // highest of the last four H1 bars  
      Low_val =Low[iLowest (NULL, PERIOD_H1,MODE_LOW,4,1)]; // lowest of the last four H1 Bars
    
      

      order_priceHigh = high_val+EntryAddPips*Point;
      order_priceLow = Low_val - EntryAddPips*Point;
     
   
   if( total < 1 && High[0] > order_priceHigh )res=OP_BUYSTOP;
   if( total < 1 && Low[0] < order_priceLow)res=OP_SELLSTOP;
 
   
//----
   return (res);   
   }  
  
//+------------------------------------------------------------------+
//| receive a trade signal                                           |
//+------------------------------------------------------------------+
// tradeDay - day of the week we trade on, if it is equal to zero, we trade every day
//
// useReversTradeDay - if it is equal to 'true', then trading days are considered as non-trading days
//
// everyTick  - if it is equal to 'true', then the function calculates a signal at every tick
//
// period - if everyTick==false, then it is calculated as soon as a new bar with this period appears
int getTradeSignal(int tradeDay,          // usually equal to 0
                   bool useReversTradeDay,// usually equal to 'false'
                   bool everyTick,        // signal is calculated at every tick
                    int period            // period of work for indicators and signals
                   )
   {
   int signal=OP_BALANCE;
//----
   if (tradeDay!=0)  // days of the week will be considered
      {
      // day we don't trade on
      if (useReversTradeDay && tradeDay==DayOfWeek()) return(signal);
      
      // trade on all days except the day equal to 'tradeDay' 
      if (!useReversTradeDay && tradeDay!=DayOfWeek()) return(signal);

      }

   if (!everyTick) // if we don't take trade signals at every tick
      { // and have no new bar at the timeframe of 'period' minutes
      if (!isNewBar(period)) return(signal); // then exit with an empty signal
      }

// Fill yourFunction() with your code/algorithm
   signal=yourFunction(period);
   
//----
   return(signal);   
   }
//+-----------------------------------------------------------------------+
//|  It returns the sign of appearance of a new bar with the given period |
//+-----------------------------------------------------------------------+
bool isNewBar(int timeFrame)
   {
   bool res=false;
   
   // array contains the open time of the current (zero) bar
   // on 7 (Seven) timeframes
   static datetime _sTime[7];  
   int i=6;

   switch (timeFrame) 
      {
      case 1  : i=0; break;
      case 5  : i=2; break;
      case 15 : i=3; break;
      case 30 : i=4; break;
      case 60 : i=5; break;
      case 240: break;
      case 1440:break;
      default:  timeFrame = 1440;
      }
//----
   if (_sTime[i]==8 || _sTime[i]!=iTime(Symbol(),timeFrame,0))
      {
      _sTime[i] = iTime(Symbol(),timeFrame,0);
      res=true;
      }
           
//----
   return(res);   
   }

//+----------------------------------------------------------------------+
//| It returns 'true', if a Global Variable with this ticket is available|
//+----------------------------------------------------------------------+
bool isOurOrder(int ticket)
   {
   bool res=false;
   
   // we don't use global variables in the test mode!
   if (IsTesting()) return(true);// immediately return a positive result

   int temp;
//----
   for (int i=0;i<5;i++)
      {
      if (GlobalVariableCheck(WindowExpertName()+"_ticket_"+i))
         {// this global variable is available
            temp=GlobalVariableGet(WindowExpertName()+"_ticket_"+i);
            if (temp==ticket)
               { // found a GV with the value equal to 'ticket'
               res=true;  // it means that the order is "friendly"
               break;
               }
         }
      }
//----
   return(res);   
   }
//+------------------------------------------------------------------+
//| It prepares an array of "friendly" orders                        |
//+------------------------------------------------------------------+
void PrepareTickets(double & arrayTickets[][9], string & comm[][2],int MN)
   {
   int count=0;   // counter of filling
   
   // let's make the array size large enough in order not to allocate memory every time
   ArrayResize(arrayTickets,20);
//----
   int total=OrdersTotal();
   for (int i=0;i<total;i++)
      {
      bool ourTicket=false;
      if (OrderSelect(i,SELECT_BY_POS))
         {
         if (isOurOrder(!OrderTicket()))
            {// if the special function has not recognized the order
            // then perform standard checks

            // checking Symbol
            if (OrderSymbol()!= Symbol()) continue;
         
            //  more checks...
            // ....
         
            // last check - MagicNumber
            if (OrderMagicNumber()!=ExpertMagicNumber) continue;
         
            }
         // we weren't stopped anywhere, so this order is "friendly"
         //  fill out the array
         arrayTickets[count][0] = OrderTicket();
         arrayTickets[count][1] = OrderType();
         arrayTickets[count][2] = OrderLots();
         arrayTickets[count][3] = OrderOpenPrice();
         arrayTickets[count][4] = OrderStopLoss();
         arrayTickets[count][5] = OrderTakeProfit();
         arrayTickets[count][6] = OrderMagicNumber();
         arrayTickets[count][7] = OrderExpiration();
         arrayTickets[count][8] = OrderOpenTime();

         comm[count][0] = OrderSymbol();
         comm[count][1] = OrderComment();
         // increase the counter of our filled orders
         count++;
         }
      }
   
   // now cut the array sizes to the minimal necessary size 
   ArrayResize(arrayTickets,count);
   //ArrayResize(comm,count);

//----
   return;   
   }

//+------------------------------------------------------------------+
//|  It calculates the level of Stop Loss                                     |
//+------------------------------------------------------------------+
double getNewSL(int    type,      // order  type we're calculating it for
                double lots,      // volume, it can be helpful
                double openPrice, // open price
                double stopLoss,  // current level of Stop Loss
                double takeProfit // current level of Take Profit
                )
   {
   double res=-1;
//----
//  here is the code to calculate Stop Loss
//----
   return(res);   
   }

//+------------------------------------------------------------------+
//|  It calculates the level of  Take Profit                         |
//+------------------------------------------------------------------+
double getNewTP(int    type,      // order  type we're calculating it for
                double lots,      // volume, it can be helpful
                double openPrice, // open price
                double stopLoss,  // current level of Stop Loss
                double takeProfit // current level of Take Profit
                )
   {
   double res=-1;
//----
//  here is the code calculating the Take Profit
//----
   return(res);   
   }

//+------------------------------------------------------------------+
//|  It calculates the new open price for a pending order            |
//+------------------------------------------------------------------+
double getNewOpenPricePending(int    type,      // order  type we're calculating it for
                              double lots,      // volume, it can be helpful
                              double openPrice, // open price
                              double stopLoss,  // current level of Stop Loss
                              double takeProfit // current level of Take Profit
                              )
   {
   double res=-1;
//----
//  here is the code calculating the open price 

//----
   return(res);   
   }

//+------------------------------------------------------------------+
//|  creates an array with the new values of SL and TP               |
//+------------------------------------------------------------------+
void CalculateSL_and_TP(bool ReversTrade,       // system reverse 
                      double arrayTickets[][9], // array of "friendly" orders
                      double & arraySL_TP[][5]  // new values of SL, TP and openPrice
                      )
   {
   // first of all, zeroize the obtained array !!
   ArrayResize(arraySL_TP,0);
//----
   int    i,size=ArrayRange(arrayTickets,0);
   int    sizeSL_TP=0;
   double lots, openPrice, SL,TP;
   double newSL,newTP, newOpen;
   double oldOpenPrice, oldSL,oldTP;    
   // of the array of orders is empty, then exit   
   if (size==0) return;

   int type,ticket,oldType;
   for (i=0;i<size;i++)
      {
      ticket    = arrayTickets[i][0]; //ticket number
      type      = arrayTickets[i][1]; //order  type
      lots      = arrayTickets[i][2]; //order volume
      openPrice = arrayTickets[i][3]; //order open price
      SL        = arrayTickets[i][4]; //Stop Loss
      TP        = arrayTickets[i][5]; //Take Profit
      
      if (ReversTrade) //  reverse all levels considering spread
         {
         switch (type)
            {
            case OP_BUY      : oldType = OP_SELL     ; break;
            case OP_SELL     : oldType = OP_BUY      ; break;
            case OP_BUYLIMIT : oldType = OP_SELLSTOP ; break;
            case OP_SELLLIMIT: oldType = OP_BUYSTOP  ; break;
            case OP_BUYSTOP  : oldType = OP_SELLLIMIT; break;
            case OP_SELLSTOP : oldType = OP_BUYLIMIT ; break;
            default: Print("Invalid order  type Type=",type," in function CalculateSL_and_TP()!!!");                           
            }

         double temp;
         int spread = MarketInfo(Symbol(),MODE_SPREAD);
         int digits = MarketInfo(Symbol(),MODE_DIGITS);
         if (type==OP_BUY || type==OP_BUYSTOP || type==OP_BUYLIMIT)  
            {
            temp = SL;
            if (TP!=0) oldSL = NormalizeDouble(TP+Point*spread,digits);
               else oldSL=0;
               
            if (SL!=0) oldTP = NormalizeDouble(temp+Point*spread,digits);
               else oldTP=0;
               
            oldOpenPrice = NormalizeDouble(openPrice - Point*spread,digits);
            }
         if (type==OP_SELL) 
            {
            temp = SL;
            if (TP!=0) oldSL = NormalizeDouble(TP-Point*spread,digits);
               else oldSL=0;
            
            if (SL!=0) oldTP = NormalizeDouble(temp-Point*spread,digits);
               else oldTP=0;
            
            oldOpenPrice = NormalizeDouble(openPrice + Point*spread,digits);
            }
         }
      else   // no system reverse
         {
         oldOpenPrice = openPrice;
         oldSL = SL;
         oldTP = TP;
         }
      
      newSL  = getNewSL(oldType,lots,oldOpenPrice,oldSL,oldTP);
      newTP  = getNewTP(oldType,lots,oldOpenPrice,oldSL,oldTP);
      
      // if it is a pending order, then get a new open price
      if (type>OP_SELL) newOpen=getNewOpenPricePending(type,lots,openPrice,oldSL,oldTP);
      
      if (newSL>0 || newTP>0 || newOpen>0)
         {
         sizeSL_TP=ArrayRange(arraySL_TP,0);
         arraySL_TP[sizeSL_TP][0] = arrayTickets[i][0]; // ticket  
         if (newSL>0) arraySL_TP[sizeSL_TP][1] = newSL; // new level of SL
            else      arraySL_TP[sizeSL_TP][1] = arrayTickets[i][4];
            
         if (newTP>0) arraySL_TP[sizeSL_TP][2] = newTP; // new level of TP
            else      arraySL_TP[sizeSL_TP][2] = arrayTickets[i][5];
         if (newOpen>0)arraySL_TP[sizeSL_TP][3] = newOpen; // new open price
            else       arraySL_TP[sizeSL_TP][3] = arrayTickets[i][3];
         if (type>OP_SELL) arraySL_TP[sizeSL_TP][4]=1; // market order
            else           arraySL_TP[sizeSL_TP][4]=0; // market order
         }
      }         
//----
   return;
   }

//+------------------------------------------------------------------+
//|  It calculates the data for opening a new order                  |
//+------------------------------------------------------------------+
void CalculateNewMarketValues(int    trSignal,
                              int    & marketType,
                              double & marketLots, 
                              double & marketSL,
                              double & marketTP,
                              string & marketcomment
                              )
   {
   // if there is no trade signal, then exit
   if (trSignal==OP_BALANCE) return;

   marketType    =-1; // this means that we won't open anything
   marketLots    = 0;
   marketSL      = 0;
   marketTP      = 0;
   marketcomment = "";


//----
   //  insert your code to calculate all parameters
   if (trSignal==OP_BUYSTOP  && StopLoss!=0) marketSL = Bid - StopLoss*Point;
   if (trSignal==OP_SELLSTOP && StopLoss!=0) marketSL = Ask + StopLoss*Point;
 
   if (trSignal==OP_BUYSTOP  && TakeProfit!=0) marketTP = Bid + TakeProfit*Point;
   if (trSignal==OP_SELLSTOP && TakeProfit!=0) marketTP = Ask - TakeProfit*Point;
 
   marketLots = Lots;
 
   if (trSignal==OP_BUYSTOP) marketType = OP_BUYSTOP;
   if (trSignal==OP_SELLSTOP) marketType = OP_SELLSTOP;
 
   marketcomment="Order uitgevoerd";
   
//----
   return;   
   }  

//+------------------------------------------------------------------+
//|  It calculates the data for placing a pending order              |
//+------------------------------------------------------------------+
void CalculateNewPendingValues(int    trSignal, 
                               int    & pendingType, 
                               double & pendingOpenPrice, 
                               double & pendingLots, 
                               double & pendingSL, 
                               double & pendingTP, 
                               string & pendingComment)
   {
   // if there is no trade signal, then exit
   if (trSignal==OP_BALANCE) return;

   pendingType      = -1; 
   pendingOpenPrice = 0; 
   pendingLots      = 0; 
   pendingSL        = 0; 
   pendingTP        = 0; 
   pendingComment   = 0;
//----
   //insert your code to calculate all parameters
//----
   return;
   } 

//+------------------------------------------------------------------+
//|  Modification of market orders                                    |
//+------------------------------------------------------------------+
void  ModifyMarkets(bool Revers,
                    bool ModifyEveryTick,
                    int  ModifyBarPeriod,
                    double newSL_and_TP[][])
   {
   int i,type,ticket,size=ArrayRange(newSL_and_TP,0);
   if (size==0) return;  // nothing to modify, exit

   bool res;
//----
   if (!ModifyEveryTick )// if every-tick modification is prohibited
      {   
      if (!isNewBar(ModifyBarPeriod)) return;   // no new bar appears
      }

   if (!Revers) // direct order of working
      {
      for (i=0;i<size;i++)
         {
         type=newSL_and_TP[i][4];
         if (type!=0) continue; // it is not a market order, skip it 
         ticket=newSL_and_TP[i][0];
         res=OrderModify(ticket,newSL_and_TP[i][3],newSL_and_TP[i][1],newSL_and_TP[i][2],0);
         if (!res)// failed to modify
            {
            Print("Order #",ticket," modification failed. Error ",GetLastError());
            // further processing the situation
            }
         }
       }  
   else  // trading system is reversed, transpose SL and TP 
      {
      for (i=0;i<size;i++)
         {
         type=newSL_and_TP[i][4];
         if (type!=0) continue; // it is not a market order, skip it 
         ticket=newSL_and_TP[i][0];
         res=OrderModify(ticket,newSL_and_TP[i][3],newSL_and_TP[i][2],newSL_and_TP[i][1],0);
         if (!res)// failed to modify
            {
            Print("Order #",ticket," modification failed. Error ",GetLastError());
            // further processing the situation
            }
         }
      }         

//----
   return;   
   }
   
//+------------------------------------------------------------------+
//|  modification of market orders                                    |
//+------------------------------------------------------------------+
void  ModifyPendings(bool Revers,
                    bool ModifyEveryTick,
                    int  ModifyBarPeriod,
                    double newSL_and_TP[][])
   {
   int i,type,ticket,size=ArrayRange(newSL_and_TP,0);
   if (size==0) return;  // nothing to modify - exit

   bool res;
//----
   if (!ModifyEveryTick )// if every-tick modification is prohibited,
      {   
      if (!isNewBar(ModifyBarPeriod)) return;   // no new bar appears
      }

   if (!Revers) // direct order of working
      {
      for (i=0;i<size;i++)
         {
         type=newSL_and_TP[i][4];
         if (type==0) continue; // it is not a pending order, skip it 
         ticket=newSL_and_TP[i][0];
         res=OrderModify(ticket,newSL_and_TP[i][3],newSL_and_TP[i][1],newSL_and_TP[i][2],0);
         if (!res)// failed to modify
            {
            Print("Order #",ticket," modification failed. Error ",GetLastError());
            // further processing the situation
            }
         }
       }  
   else  // trading system is reversed, transpose SL and TP 
      {
      for (i=0;i<size;i++)
         {
         type=newSL_and_TP[i][4];
         if (type==0) continue; // it is not a pending order, skip it 
         ticket=newSL_and_TP[i][0];
         res=OrderModify(ticket,newSL_and_TP[i][3],newSL_and_TP[i][2],newSL_and_TP[i][1],0);
         if (!res)// failed to modify
            {
            Print("Order #",ticket," modification failed. Error ",GetLastError());
            // further processing the situation
            }
         }
      }         

//----
   return;   
   }
   
//+------------------------------------------------------------------+
//|  prepare an array of tickets for closing                         |
//+------------------------------------------------------------------+
void PrepareTicketsToClose(int signal, bool Revers, int & ticketsClose[][2], double & lots[],double arrayTickets[][9])
   {
   int size=ArrayRange(arrayTickets,0);
//----
   if (size==0) return;

   int i,type,ticket,closeSize;
   for (i=0;i<size;i++)
      {
      type=arrayTickets[i][1];
      // if order type is not a market order, then skip it
      if (type>OP_SELL) continue;
      
      if (Revers) // reverse the type of a market order
         {
         if (type==OP_BUY) type=OP_SELL; else type=OP_BUY;
         }

      // here we will decide the fate of each open order
      //  whether to keep it open or to add it to the array of orders to be closed
      if (type==OP_BUY)
         {
         //  
         // code that allows us to retain buying
         
         // as an example
         if (signal==OP_BUYSTOP) continue;
         }
      
      if (type==OP_SELL)
         {
         //  
         // code that allows us to retain selling
         
         // as an example
         if (signal==OP_SELLSTOP) continue;
         }

      closeSize=ArrayRange(ticketsClose,0);
      ArrayResize(ticketsClose,closeSize+1);
      ArrayResize(lots,closeSize+1);
      ticketsClose[closeSize][0] = arrayTickets[i][0]; // ticket #
      ticketsClose[closeSize][1] = arrayTickets[i][1]; // order  type
      
      // here we will specify the amount of lots to be closed
      lots[closeSize] = arrayTickets[i][2]; // volume to be closed
      // you can close them partly, then you should rewrite the upper line
      }
//----
   return;   
   }

//+------------------------------------------------------------------+
//|  prepare an array of tickets for deletion of pending orders      |
//+------------------------------------------------------------------+
void PrepareTicketsToDelete(int signal, bool Revers, int & ticketsDelete[],double arrayTickets[][9])
   {
   int size=ArrayRange(arrayTickets,0);
//----
   if (size==0) return;
   ArrayResize(ticketsDelete,0);

   int i,type,ticket,deleteSize;
   for (i=0;i<size;i++)
      {
      type=arrayTickets[i][1];
      // if the order type is not a pending order, then skip it
      if (type<=OP_SELL) continue;
            
      if (Revers) // reverse the type of a market order
         {
         switch (type)
            {
            case OP_BUYLIMIT : type = OP_SELLSTOP ; break;
            case OP_SELLLIMIT: type = OP_BUYSTOP  ; break;
            case OP_BUYSTOP  : type = OP_SELLLIMIT; break;
            case OP_SELLSTOP : type = OP_BUYLIMIT ; break;
            }
         }

      // here we decide the fate for each pending order
      // whether to retain it or add to the array of orders to be deleted
      // here there is an example of buying signal retains 
      // of pending orders OP_BUYLIMIT and OP_BUYSTOP
      // the same applies to selling signals
      if (type==OP_BUYLIMIT || OP_BUYSTOP)
         {
         //  
         // code that allows us to retain buying
         // as an example
         if (signal==OP_BUY) continue;
         }
      
      if (type==OP_SELLLIMIT || OP_SELLSTOP)
         {
         //  
         // code that allows us to retain selling
         // as an example
         if (signal==OP_SELL) continue;
         }

      deleteSize=ArraySize(ticketsDelete);
      ArrayResize(ticketsDelete,deleteSize+1);
      ticketsDelete[deleteSize] = arrayTickets[i][0];
      }
//----
   return;   
   }
//+------------------------------------------------------------------+
//|  It closes orders with the given tickets                         |
//+------------------------------------------------------------------+
void CloseMarketOrders(int ticketsArray[][2], double lotsArray[])
   {  
//----
   int i,size=ArrayRange(ticketsArray,0);
   if (size==0) return;
   
   int ticket,type;
   double lots;
   bool res;
   
   int total=OrdersTotal(); 
   
   for (i=0;i<size;i++)
      {
      ticket = ticketsArray[i][0];
      type   = ticketsArray[i][1];
      lots   = lotsArray[i];
      RefreshRates(); // just in case, update the information of the market environment

      // block to close buying trades
      if (type==OP_BUY)
         {
         res = OrderClose(ticket,lots,Bid,Slippage,Orange);
         if (!res)
            {
            Print("Failed to open buying order #",ticket,"!  Error #",GetLastError());
            //  further processing the error, code it yourself, please
            }
         }

      //  block to close selling trades
      if (type==OP_SELL)
         {
         res = OrderClose(ticket,lots,Ask,Slippage,Orange);
         if (!res)
            {
            Print("Failed to open selling order #",ticket,"!  Error #",GetLastError());
            //  further processing the error, code it yourself, please
            }
         }

      } 
//----
   return;
   }

//+------------------------------------------------------------------+
//|  It deletes pending orders with the given tickets                |
//+------------------------------------------------------------------+
void DeletePendingOrders(int ticketsArray[])
   {  
//----
   int i,size=ArraySize(ticketsArray);
   if (size==0) return;
   
   int ticket;
   bool res;

   // block to delete pending orders
   for (i=0;i<size;i++)
      {
      ticket = ticketsArray[i];
      res = OrderDelete(ticket);
      if (!res)
         {
         Print("Failed to delete pending order #",ticket,"!  Error #",GetLastError());
         //  further processing the error, code it yourself, please
         }
      } 
//----
   return;
   }

//+------------------------------------------------------------------+
//|  It opens a position by market                                   |
//+------------------------------------------------------------------+
void OpenMarketOrder(bool   reversTrade,// sign of system reverse
                     int    Type,       // order  type - OP_BUY or OP_SELL
                     double lots,       // volume of the position to be opened
                     double SL,         // level of Stop Loss
                     double TP,         // level of TakeProfit
                     string comment)    // comment on the order
   {
   int openType;
   
   if (reversTrade)                       // reverse signals
      {
      double temp;
      int spread = MarketInfo(Symbol(),MODE_SPREAD);
      int digits = MarketInfo(Symbol(),MODE_DIGITS);
      if (Type==OP_BUY)  
         {
         openType = OP_SELL; // buying will be converted into selling
         temp = SL;
         if (TP!=0) SL = NormalizeDouble(TP+Point*spread,digits);
            else SL=0;
         if (temp!=0) TP = NormalizeDouble(temp+Point*spread,digits);
            else TP=0;
         }
      if (Type==OP_SELL) 
         {
         openType = OP_BUY;  // selling will be converted into buying
         temp = SL;
         if (TP!=0) SL = NormalizeDouble(TP-Point*spread,digits);
            else SL=0;
         if (temp!=0) TP = NormalizeDouble(temp-Point*spread,digits);
            else TP=0;
         }
      }
   else
      {
      openType=Type;
      }   
//----

   if (lots==0) return;
   
   RefreshRates();
   double price;
   color Color;
   if (openType==OP_BUYSTOP) 
      {
      price = High[1]+ HighPlus*Point;
      Color = Blue; 
      }
   if (openType==OP_SELLSTOP) 
      {
      price=Low[1]-LowPlus*Point;
      Color = Red; 
      }
   bool ticket = OrderSend(Symbol(),openType,lots,price,Slippage,SL,TP,comment,ExpertMagicNumber,0,Color); 
   if (ticket<0)
      {
      Print("Failed to open a position by market");
      Print("Type=",openType,"  lots=",lots,"  SL=",SL,"  TP=",TP," comment=",comment,"  ExpertMagicNumber=",ExpertMagicNumber);
      //  further processing the situation, code it yourself, please
      }
//----
   return;
   }

//+------------------------------------------------------------------+
//|  It places a pending order                                       |
//+------------------------------------------------------------------+
void SendPendingOrder(bool   reversTrade,// sign of system reverse
                      int    Type,
                      double OpenPrice, 
                      double Lots, 
                      double SL, 
                      double TP,
                      string comment)
   {
   if (Type==-1) return; 
   
   if (reversTrade)                       // reverse signals
      {
      double temp;
      int spread = MarketInfo(Symbol(),MODE_SPREAD);
      int digits = MarketInfo(Symbol(),MODE_DIGITS);

      if (Type==OP_BUYLIMIT || Type==OP_BUYSTOP)
         {
         OpenPrice = NormalizeDouble(OpenPrice - spread*Point,digits);
         temp=SL;
         SL = NormalizeDouble(TP+Point*spread,digits);
         TP = NormalizeDouble(temp+Point*spread,digits);
         }
      if (Type==OP_SELLLIMIT || Type==OP_SELLSTOP)
         {
         OpenPrice = NormalizeDouble(OpenPrice + spread*Point,digits);
         temp=SL;
         SL = NormalizeDouble(TP - Point*spread,digits);
         TP = NormalizeDouble(temp - Point*spread,digits);
         }

      switch (Type)
         {
         case OP_BUYLIMIT:  Type = OP_SELLSTOP ; break;
         case OP_SELLLIMIT: Type = OP_BUYSTOP  ; break;
         case OP_BUYSTOP:   Type = OP_SELLLIMIT; break;
         case OP_SELLSTOP:  Type = OP_BUYLIMIT;  break;
         default: Print("Invalid order type Type=",Type," in function SendPendingOrder()!!!");                           
         
         }
      }
   color Color;
   if (Type==OP_SELLLIMIT || Type==OP_SELLSTOP) Color = Red;
   if (Type==OP_BUYLIMIT || Type==OP_BUYSTOP)   Color = Blue;

   bool res = OrderSend(Symbol(),Type,Lots,OpenPrice,Slippage,SL,TP,comment,ExpertMagicNumber,0,Color); 
   if (!res)
      {
      Print("Failed to place a pending order");
      //  further processing the situation, code it yourself, please
      }


//----

//----
   return;
   }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   // always zeroize the array size before the first use of it
   ArrayResize(Tickets,0);
   //ArrayResize(CommentsTicket,0);
   
   // get the arrays of "friendly" orders
   PrepareTickets(Tickets,CommentsTicket,ExpertMagicNumber);
   
//----

/**
      1. Trade Signals. Receiving trade signals
      a) Every tick                       (TradeSignalEveryTick=true)
      b) Each bar of the given period     (TradeSignalBarPeriod=...)
      OP_BUY      - to buy
      OP_SELL     - to sell
      OP_BALANCE  - no signal
*/


   int trSignal=getTradeSignal(TradeDay,ReversDay,
                       TradeSignalEveryTick,TradeSignalBarPeriod);
                       
/**
      2. 
        a) Calculate SL and TP for each open order
        b) Calculate OpenPrice, SL, TP, and Lots for a new order
        c) Calculate OpenPrice, SL and TP for each pending order
*/

   CalculateSL_and_TP(ReversTradeSignal,Tickets,newSL_and_TP);


   double marketLots,marketSL,marketTP;
   int marketType, pendingType;
   string marketComment, pendingComment;
   double pendingOpenPrice, pendingLots, pendingSL, pendingTP;

   CalculateNewMarketValues(trSignal, marketType, marketLots,marketSL,marketTP,marketComment);
   CalculateNewPendingValues(trSignal, pendingType, pendingOpenPrice, pendingLots, pendingSL, pendingTP, pendingComment);

/**
      3. 
        a) Modification of each open order for every tick (SL and TP)        
               (ModifyMarketOrderEveryTick = true)
               
        b) Modification of each pending order for every tick (OpenPrice, SL and TP)
               (ModifyPendingEveryTick = true)
        
        c) Modification of each open order on every new bar of the selected period (SL and TP)
               (ModifyMarketBarPeriod = ...)
        
        d) Modification of each pending order on every new bar of the selected period (OpenPrice, SL and TP)
               (ModifyPendingBarPeriod = ...)
        
*/

   ModifyMarkets(ReversTradeSignal,ModifyMarketOrderEveryTick,ModifyMarketBarPeriod,newSL_and_TP);
   ModifyPendings(ReversTradeSignal,ModifyPendingEveryTick,ModifyPendingBarPeriod,newSL_and_TP);

/**
      4. 
        a) Close an open order by time
        b) Close an open order by signal
*/
   // tickets of orders to be closed
   int ticketsToClose[][2];
   double lotsToClose[]; 
   
   // zeroize array sizes
   ArrayResize(ticketsToClose,0);
   ArrayResize(lotsToClose,0);
   
   // prepare an array of tickets to be closed
   if (trSignal!=OP_BALANCE) PrepareTicketsToClose(trSignal,ReversTradeSignal,ticketsToClose,lotsToClose,Tickets);
   
   // close the specified orders
   CloseMarketOrders(ticketsToClose,lotsToClose);

/**
      5. 
        a) Delete a pending order by time
        b) Delete a pending order by condition
*/
   // tickets of orders to be deleted
   int ticketsToDelete[];

   // prepare an array of tickets to be deleted
   if (trSignal!=OP_BALANCE) PrepareTicketsToDelete(trSignal,ReversTradeSignal,ticketsToDelete,Tickets);

   // delete the specified orders
   DeletePendingOrders(ticketsToDelete);

/**
      6. 
        a) Open an order by market
        b) Place a pending order without expiration
        c) Place a pending order with expiration
*/

   if (trSignal!=OP_BALANCE) OpenMarketOrder(ReversTradeSignal,marketType,marketLots,marketSL,marketTP,marketComment);
   if (trSignal!=OP_BALANCE) SendPendingOrder(ReversTradeSignal,pendingType,pendingOpenPrice, pendingLots, pendingSL, pendingTP,pendingComment);
//----
   return(0);
  }
 
//+------------------------------------------------------------------+